/*
 * Copyright (c) 2016 deltaDNA Ltd. All rights reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.deltadna.android.sdk

import com.google.common.truth.Truth.assertThat
import org.junit.Test
import org.junit.runner.RunWith
import org.junit.runners.JUnit4

@RunWith(JUnit4::class)
class EventTest {
    
    @Test(expected = IllegalArgumentException::class)
    fun nameCannotBeNull() {
        KEvent(null)
    }
    
    @Test(expected = IllegalArgumentException::class)
    fun nameCannotBeEmpty() {
        KEvent("")
    }
    
    @Test
    fun name() {
        assertThat(KEvent("name").name).isEqualTo("name")
    }
    
    @Test(expected = IllegalArgumentException::class)
    fun paramsCannotBeNull() {
        KEvent("name", null)
    }
    
    @Test
    fun params() {
        with(Params()) {
            assertThat(KEvent("name", this).params).isSameAs(this)
        }
    }
    
    @Test
    fun putParam() {
        with(KEvent("name")) {
            putParam("key", "value")
            
            assertThat(params.json.get("key")).isEqualTo("value")
            
            val params = Params()
            params.put("param", "value")
            putParam("params", params)
            
            assertThat(this.params.json.getJSONObject("params").get("param"))
                    .isEqualTo("value")
        }
    }
    
    private class KEvent : Event<KEvent> {
        constructor(name: String?) : super(name)
        constructor(name: String?, params: Params?) : super(name, params)
    }
}
